local L10N = PeskyUF.L10N

-- (LibIconDB.Tooltip) => Unit
local function handleMinimapTooltip(tooltip)

  local timeStr = nil

  if Pesky.secondsElapsed ~= nil then
    timeStr = Pesky.secondsToTimeStr(Pesky.targetSeconds - Pesky.secondsElapsed) .. " " .. L10N["remaining"]
  else
    timeStr = L10N["Timer not yet started"]
  end

  tooltip:SetText("Pesky: " .. timeStr)
  tooltip:Show()

end

-- () => Unit
local function showMenu()

  local options =
    { { text=L10N["Pesky"              ], notCheckable=true, isTitle=true                     }
    , { text=L10N["Reset Timer"        ], notCheckable=true, func=(function() Pesky.setTimer() end) }
    , { text=L10N["Take a Break"       ], notCheckable=true, func=Pesky.startBreak                  }
    , { text=L10N["Pause Timer"        ], notCheckable=true, func=Pesky.pause                       }
    , { text=L10N["Unpause Timer"      ], notCheckable=true, func=Pesky.unpause                     }
    , { text=L10N["Show Time Remaining"], notCheckable=true, func=Pesky.remaining                   }
    , { text=L10N["Configure"          ], notCheckable=true, func=Pesky.config                      }
    }

  EasyMenu(options, PeskyMinimapContextMenu, "cursor", 0, 0, "MENU")

end

-- (Table, String) => Unit
local function handleClick(self, button)
  if button == "LeftButton" then
    Pesky.config()
  elseif button == "RightButton" then
    showMenu()
  end
end

local ldbConfig =
  {
    type = "data source",
    text = "PeskyIcon",
    icon = "Interface\\Icons\\INV_Misc_Pocketwatch_02",
    OnTooltipShow = handleMinimapTooltip,
    OnClick = handleClick,
  }

Pesky.ldb = LibStub("LibDataBroker-1.1"):NewDataObject("PeskyIcon", ldbConfig)

Pesky.icon = LibStub("LibDBIcon-1.0")

-- (String) => Unit
function Pesky.changeMinimapIcon(newPath)
  Pesky.icon:GetMinimapButton("PeskyIcon").icon:SetTexture(newPath)
end

-- (Number, Number, Number) => Unit
function Pesky.changeMinimapIconRGB(r, g, b)
  Pesky.icon:GetMinimapButton("PeskyIcon").icon:SetTexture()
  Pesky.icon:GetMinimapButton("PeskyIcon").icon:SetColorTexture(r, g, b)
end
