local L10N = PeskyUF.L10N

Pesky.secondsElapsed = nil
Pesky.targetSeconds  = nil
Pesky.isPaused       = nil

local ticker = nil

local timers = {}

-- () => Unit
local function alarmCallback()
  Pesky.changeMinimapIconRGB(math.random(), math.random(), math.random())
end

-- () => Unit
local function activateAlarm()
  PlaySound(SOUNDKIT.READY_CHECK)
  ticker:Cancel()
  ticker = C_Timer.NewTicker(1, alarmCallback)
  alarmCallback()
end

-- () => Unit
local function tickerCallback()
  if Pesky.secondsElapsed ~= nil and not Pesky.isPaused then
    Pesky.secondsElapsed = Pesky.secondsElapsed + 0.1
    if Pesky.secondsElapsed >= Pesky.targetSeconds then
      Pesky.secondsElapsed = nil
      activateAlarm()
    else
      local progress = Pesky.secondsElapsed / Pesky.targetSeconds
      Pesky.changeMinimapIconRGB(progress, 1 - progress, 0)
    end
  end
end

-- () => Unit
local function endBreakCallback()
  Pesky.changeMinimapIconRGB(0, 0, 1)
  PlaySound(SOUNDKIT.GS_CHARACTER_SELECTION_ENTER_WORLD)
  local seconds = Pesky.timeStrToSeconds(Pesky.db.profile.defaultGracePeriod)
  local timer   = C_Timer.NewTimer(seconds, function() Pesky.setTimer(Pesky.targetSeconds) end)
  table.insert(timers, timer)
end

-- (Number?) => Unit
function Pesky.setTimer(seconds)

  seconds = seconds or Pesky.timeStrToSeconds(Pesky.db.profile.defaultTimerPeriod)

  for i, timer in ipairs(timers) do
    timer:Cancel()
  end

  timers               = {}
  Pesky.targetSeconds  = seconds
  Pesky.secondsElapsed = 0
  Pesky.isPaused       = true

  Pesky.unpause()

end

-- () => Unit
function Pesky.startBreak()
  ticker:Cancel()
  local seconds = Pesky.timeStrToSeconds(Pesky.db.profile.defaultBreakPeriod)
  local timer   = C_Timer.NewTimer(seconds, endBreakCallback)
  table.insert(timers, timer)
  Pesky.changeMinimapIconRGB(1, 0, 0)
end

-- () => Unit
function Pesky.pause()
  if Pesky.secondsElapsed ~= nil then
    if not Pesky.isPaused then
      ticker:Cancel()
      Pesky.isPaused = true
    else
      print(L10N["Timer already paused"])
    end
  else
    print(L10N["Timer not yet started"])
  end
end

-- () => Unit
function Pesky.unpause()
  if Pesky.secondsElapsed ~= nil then
    if Pesky.isPaused then

      Pesky.isPaused = false

      if ticker ~= nil then
        ticker:Cancel()
      end

      ticker = C_Timer.NewTicker(0.1, tickerCallback)
      tickerCallback()

    else
      print(L10N["Timer already running"])
    end
  else
    print(L10N["Timer not yet started"])
  end
end

-- () => Unit
function Pesky.remaining()
  if Pesky.secondsElapsed ~= nil then
    print("Pesky: " .. Pesky.secondsToTimeStr(Pesky.targetSeconds - Pesky.secondsElapsed))
  else
    print(L10N["Timer not yet started"])
  end
end

-- () => Unit
function Pesky.config()
  if PeskyConfigFrame:IsVisible() then
    PeskyConfigFrame:Hide()
  else
    PeskyConfigFrame:Show()
  end
end

BINDING_HEADER_PESKY           = L10N["Pesky"]
BINDING_NAME_RESETTIMER        = L10N["Reset Timer"]
BINDING_NAME_TAKEABREAK        = L10N["Take a Break"]
BINDING_NAME_PAUSETIMER        = L10N["Pause Timer"]
BINDING_NAME_UNPAUSETIMER      = L10N["Unpause Timer"]
BINDING_NAME_SHOWTIMEREMAINING = L10N["Show Time Remaining"]
